/**
 * This file is part of Carma Launcher.
 * 
 * Carma Launcher is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * 
 * Carma Launcher is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * Carma Launcher. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2011, Casper Kejlberg-Rasmussen.
 */
package carmalauncher;

import javax.swing.JFrame;
import masterserver.MasterServer;

/**
 *
 * @author Casper Kejlberg-Rasmussen
 */
public class CarmaLauncher {
    private static int masterPort = 14390;
    private static int cleanupPort = 14391;
    private static int cleanUpInterval = 300;

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        boolean runAsMaster = false;
        boolean runAsClient = true;
        if(args.length >= 1){
            try {
                if (args[0].equals("-masterserver")) {
                    runAsMaster = true;
                    runAsClient = false;
                    if (args.length == 2) {
                        masterPort = Integer.parseInt(args[1]);
                    }
                    if (args.length == 3) {
                        masterPort = Integer.parseInt(args[1]);
                        cleanUpInterval = Integer.parseInt(args[2]);
                    }
                } else if (args.length == 1) {
                    runAsMaster = true;
                    runAsClient = false;
                    cleanupPort = Integer.parseInt(args[0]);
                }
            } catch (NumberFormatException ex) {
                System.out.println("CarmaLauncher, Copyright 2011, Casper Kejlberg-Rasmussen.");
                System.out.println("Usage: java -jar CarmaLauncher.jar [cleanUpPort]");
                System.out.println("Usage: java -jar CarmaLauncher.jar -masterserver [masterPort] [cleanUpInterval]");
                System.out.println();
                System.out.println(" -masterserver:     If this flag is enabled the masterserver will run, else the\n"
                                 + "                    client with GUI will run, for launching singleplayer and\n"
                                 + "                    multiplayer games.");
                System.out.println();
                System.out.println(" [cleanUpPort]:     The port number to use to contact servers/clients to check if\n"
                                 + "                    they are alive, else they not be listed in the masterserver.");
                System.out.println();
                System.out.println(" [masterPort]:      The port number where the masterserver listens for request on.");
                System.out.println();
                System.out.println(" [cleanUpInterval]: The time interval before the master server will go through the\n"
                                 + "                    serverlist and remove servers/clients that are not alive anymore.");
                System.exit(0);
            }
        }
        if(runAsMaster){
            server();
        }
        if(runAsClient){
            client();
        }
    }
    
    public static void server(){
        MasterServer ms = new MasterServer(masterPort, cleanupPort, cleanUpInterval);
        ms.startServer();
    }
    
    public static void client(){
        JFrame mw = new MainWindow();
        mw.setTitle("Carma Launcher");
        mw.setLocationRelativeTo(null);
        mw.setVisible(true);
        AliveThread at = new AliveThread(cleanupPort);
        Thread t = new Thread(at);
        t.start();
    }
}
