/**
 * This file is part of Carma Launcher.
 * 
 * Carma Launcher is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * 
 * Carma Launcher is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * Carma Launcher. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2011, Casper Kejlberg-Rasmussen.
 */
package carmalauncher;

import java.awt.event.ItemEvent;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Map;
import java.util.TreeMap;
import javax.swing.JFrame;

/*
 * MainWindow.java
 *
 * Created on 10-06-2011, 22:37:14
 */
/**
 *
 * @author Casper Kejlberg-Rasmussen
 */
public class MainWindow extends javax.swing.JFrame {

    /** Creates new form MainWindow */
    public MainWindow() {
        initComponents();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        mutiplayerButton = new javax.swing.JButton();
        singleplayerButton = new javax.swing.JButton();
        optionsButton = new javax.swing.JButton();
        splatPackCheckBox = new javax.swing.JCheckBox();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);

        mutiplayerButton.setText("Multiplayer");
        mutiplayerButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                mutiplayerButtonMouseClicked(evt);
            }
        });

        singleplayerButton.setText("Singleplayer");
        singleplayerButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                singleplayerButtonMouseClicked(evt);
            }
        });

        optionsButton.setText("Options");
        optionsButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                optionsButtonMouseClicked(evt);
            }
        });

        splatPackCheckBox.setText("Play Splat Pack");
        splatPackCheckBox.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                splatPackCheckBoxItemStateChanged(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGap(94, 94, 94)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addGap(10, 10, 10)
                        .addComponent(splatPackCheckBox))
                    .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                        .addComponent(optionsButton, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 134, Short.MAX_VALUE)
                        .addComponent(mutiplayerButton, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 134, Short.MAX_VALUE)
                        .addComponent(singleplayerButton, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.PREFERRED_SIZE, 134, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap(103, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGap(45, 45, 45)
                .addComponent(singleplayerButton, javax.swing.GroupLayout.PREFERRED_SIZE, 37, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(mutiplayerButton, javax.swing.GroupLayout.PREFERRED_SIZE, 37, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(optionsButton, javax.swing.GroupLayout.PREFERRED_SIZE, 37, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(splatPackCheckBox)
                .addContainerGap(67, Short.MAX_VALUE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private boolean playSplatPack = false;
    
    private String getConfFile() throws IOException {
        if (playSplatPack) {
            return "CarmageddonSplatPack.conf";
        } else {
            return "Carmageddon.conf";
        }
    }
    
    protected Map<String, String> getFromConf(String[] options) throws FileNotFoundException, IOException{
        BufferedReader br = new BufferedReader(new FileReader(new File((new File(".")).getCanonicalPath() + "/DOSBox-0.74/" + getConfFile())));
        String line = br.readLine();
        Map<String, String> res = new TreeMap<String,String>();
        while (line != null) {
            for(String option : options){
                if (line.contains(option + "=")) {
                    res.put(option, line.substring(line.indexOf("=") + 1));
                }
            }
            line = br.readLine();
        }
        br.close();
        return res;
    }
    
    protected void saveToConf(Map<String, String> optionPairs) throws FileNotFoundException, IOException{
        BufferedReader br = new BufferedReader(new FileReader(new File((new File(".")).getCanonicalPath() + "/DOSBox-0.74/" + getConfFile())));
        String modifiedConf = "";
        String line = br.readLine();
        while (line != null) {
            boolean matched = false;
            for(String option : optionPairs.keySet()){
                if (line.contains(option + "=")) {
                    modifiedConf += option + "=" + optionPairs.get(option) + "\n";
                    matched = true;
                }
            }
            if(!matched) {
                modifiedConf += line + "\n";
            }
            line = br.readLine();
        }
        br.close();
        BufferedWriter bw = new BufferedWriter(new FileWriter(new File((new File(".")).getCanonicalPath() + "/DOSBox-0.74/" + getConfFile())));
        bw.write(modifiedConf);
        bw.flush();
        bw.close();
    }
    
    protected void saveChangeToConf(boolean isClient, boolean isServer, String ip){
        try {
            BufferedReader br = new BufferedReader(new FileReader(new File((new File("."))
                    .getCanonicalPath() + "/DOSBox-0.74/" + getConfFile())));
            String modifiedConf = "";
            String line = br.readLine();
            while(line != null){
                if(line.contains("IPXNET STARTSERVER")){
                    if(isServer){
                        modifiedConf += "IPXNET STARTSERVER" + "\n";
                    } else {
                        modifiedConf += "#IPXNET STARTSERVER" + "\n";
                    }
                } else if(line.contains("IPXNET CONNECT")){
                    if(isClient){
                        modifiedConf += "IPXNET CONNECT " + ip + "\n";
                    } else {
                        modifiedConf += "#IPXNET CONNECT" + "\n";
                    }
                } else {
                    modifiedConf += line + "\n";
                }
                line = br.readLine();
            }
            BufferedWriter bw = new BufferedWriter(new FileWriter(new File((new File("."))
                    .getCanonicalPath() + "/DOSBox-0.74/" + getConfFile())));
            bw.write(modifiedConf);
            bw.flush();
            bw.close();
        } catch (IOException ex) {
            JFrame ew = new ErrorWindow(ex.getMessage());
            ew.setTitle("Error");
            ew.setLocationRelativeTo(this);
            ew.setVisible(true);
            ew.setAlwaysOnTop(true);
        }
    }
    
    protected Process launchGame() throws IOException{
        String currentDir = (new File(".")).getCanonicalPath();
        Process proc = Runtime.getRuntime().exec(new String[]{currentDir + "/DOSBox-0.74/DOSBox.exe",
            "-conf", getConfFile()}, null, new File(currentDir + "/DOSBox-0.74/"));
        return proc;
    }
    
    private void singleplayerButtonMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_singleplayerButtonMouseClicked
        try {
            saveChangeToConf(false, false, "");
            launchGame();
        } catch (IOException ex) {
            JFrame ew = new ErrorWindow(ex.getMessage());
            ew.setTitle("Error");
            ew.setLocationRelativeTo(this);
            ew.setVisible(true);
            ew.setAlwaysOnTop(true);
        }
    }//GEN-LAST:event_singleplayerButtonMouseClicked

    private void mutiplayerButtonMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_mutiplayerButtonMouseClicked
        JFrame mpw = new MultiplayerWindow(playSplatPack, this);
        mpw.setTitle("Multiplayer");
        mpw.setLocationRelativeTo(this);
        mpw.setVisible(true);
    }//GEN-LAST:event_mutiplayerButtonMouseClicked

    private void optionsButtonMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_optionsButtonMouseClicked
        try {
            JFrame ow;
            ow = new OptionsWindow(this);
            ow.setTitle("Options");
            ow.setLocationRelativeTo(this);
            ow.setVisible(true);
        } catch (IOException ex) {
            JFrame ew = new ErrorWindow(ex.getMessage());
            ew.setTitle("Error");
            ew.setLocationRelativeTo(this);
            ew.setVisible(true);
            ew.setAlwaysOnTop(true);
        }
    }//GEN-LAST:event_optionsButtonMouseClicked

    private void splatPackCheckBoxItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_splatPackCheckBoxItemStateChanged
        playSplatPack = (evt.getStateChange() == ItemEvent.SELECTED);
    }//GEN-LAST:event_splatPackCheckBoxItemStateChanged

    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton mutiplayerButton;
    private javax.swing.JButton optionsButton;
    private javax.swing.JButton singleplayerButton;
    private javax.swing.JCheckBox splatPackCheckBox;
    // End of variables declaration//GEN-END:variables
}
