/**
 * This file is part of Carma Launcher.
 * 
 * Carma Launcher is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * 
 * Carma Launcher is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * Carma Launcher. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2011, Casper Kejlberg-Rasmussen.
 */

/*
 * MultiplayerWindow.java
 *
 * Created on 10-06-2011, 23:05:16
 */
package carmalauncher;

import java.awt.event.ItemEvent;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.math.BigInteger;
import java.net.InetAddress;
import java.net.Socket;
import java.util.Map;
import javax.swing.JFrame;
import javax.swing.table.DefaultTableModel;
import masterserver.RequestType;
import masterserver.ServerInfo;

/**
 *
 * @author Casper Kejlberg-Rasmussen
 */
public class MultiplayerWindow extends javax.swing.JFrame {
    private boolean isSplatPack;
    private MainWindow mainWindow;
    private InetAddress masterServerIP;
    private int masterServerPort = 14390;
    private boolean isPrivateGame = false;
    private Map<BigInteger, ServerInfo> serverlist;
    
    /** Creates new form MultiplayerWindow */
    public MultiplayerWindow(boolean isSplatPack, MainWindow mainWindow) {
        this.isSplatPack = isSplatPack;
        this.mainWindow = mainWindow;
        initComponents();
        try {
            String msIpPort;
            msIpPort = mainWindow.getFromConf(new String[]{"#masterserver"}).get("#masterserver");
            masterServerIP = InetAddress.getByName(msIpPort);
            if(msIpPort.contains(":")){
                masterServerPort = Integer.parseInt(msIpPort.substring(msIpPort.indexOf(":")+1));
            }
            // Retrieves the serverlist
            Socket socket = new Socket(masterServerIP, masterServerPort);
            ServerInfo si = new ServerInfo("", RequestType.RetrieveServerList);
            ObjectOutputStream oos = new ObjectOutputStream(socket.getOutputStream());
            oos.writeObject(si);
            oos.flush();
            ObjectInputStream ois = new ObjectInputStream(socket.getInputStream());
            serverlist = (Map<BigInteger, ServerInfo>) ois.readObject();
            oos.close();
            ois.close();
            socket.close();
            int row = 0;
            // Populates the serverlist window with rows
            DefaultTableModel model = new DefaultTableModel(new Object[]{"IP", "Server type", "Players"}, 0);
            serverListTable.setModel(model);
            for(ServerInfo s : serverlist.values()){
                Object[] data = new Object[3];
                data[0] = s.getServerIP().toString().substring(1);
                data[1] = s.getGametype().toString();
                data[2] = (s.getClients().size()+1) + "";
                model.addRow(data);
                row++;
            }
        } catch (IOException ex) {
            JFrame ew = new ErrorWindow("Could not contact the master server\n" + ex.getMessage());
            ew.setTitle("Error");
            ew.setLocationRelativeTo(this);
            ew.setVisible(true);
            ew.setAlwaysOnTop(true);
        } catch (ClassNotFoundException ex){
            JFrame ew = new ErrorWindow("Error communication with the master server\n" + ex.getMessage());
            ew.setTitle("Error");
            ew.setLocationRelativeTo(this);
            ew.setVisible(true);
            ew.setAlwaysOnTop(true);
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        hostButton = new javax.swing.JButton();
        joinButton = new javax.swing.JButton();
        privateGameCheckBox = new javax.swing.JCheckBox();
        joinIPTextField = new javax.swing.JTextField();
        jScrollPane1 = new javax.swing.JScrollPane();
        serverListTable = new javax.swing.JTable();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);

        hostButton.setText("Host");
        hostButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                hostButtonMouseClicked(evt);
            }
        });

        joinButton.setText("Join");
        joinButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                joinButtonMouseClicked(evt);
            }
        });

        privateGameCheckBox.setText("Private game");
        privateGameCheckBox.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                privateGameCheckBoxItemStateChanged(evt);
            }
        });

        joinIPTextField.setText("enter server ip or url");

        serverListTable.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "IP", "Game type", "Player count"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.String.class, java.lang.Integer.class
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }
        });
        serverListTable.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                serverListTableMouseClicked(evt);
            }
        });
        jScrollPane1.setViewportView(serverListTable);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 486, Short.MAX_VALUE)
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, layout.createSequentialGroup()
                        .addComponent(privateGameCheckBox, javax.swing.GroupLayout.PREFERRED_SIZE, 108, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(hostButton, javax.swing.GroupLayout.PREFERRED_SIZE, 67, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(joinIPTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 239, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(joinButton)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(privateGameCheckBox)
                    .addComponent(joinIPTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(joinButton)
                    .addComponent(hostButton))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 262, Short.MAX_VALUE)
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents
  
    private void talkToMasterServer(String gametype, RequestType requestType) throws IOException{
        Socket socket = new Socket(masterServerIP, masterServerPort);
        ServerInfo si = new ServerInfo(gametype, requestType);
        ObjectOutputStream oos = new ObjectOutputStream(socket.getOutputStream());
        oos.writeObject(si);
        oos.flush();
        oos.close();
        socket.close();
    }
    
    private void joinButtonMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_joinButtonMouseClicked
        try {
            // Only publish the join of the joined server if it is in the serverlist
            boolean isPrivate = true;
            for(ServerInfo si : serverlist.values()){
                String ip = joinIPTextField.getText().contains(":")
                        ? joinIPTextField.getText().substring(0,joinIPTextField.getText().indexOf(":"))
                        : joinIPTextField.getText();
                if(si.getServerIP().equals(InetAddress.getByName(ip))){
                   isPrivate = false; 
                }
            }
            if(!isPrivate){
                talkToMasterServer(isSplatPack ? "SP" : "C1", RequestType.JoinServer);
            }
            // Launch multiplayer and connect to the ip in joinIPTextField
            mainWindow.saveChangeToConf(true, false, joinIPTextField.getText());
            Process proc = mainWindow.launchGame();
            proc.waitFor();
            if(!isPrivate){
                talkToMasterServer(isSplatPack ? "SP" : "C1", RequestType.LeaveServer);
            }
        } catch (Exception ex){
            JFrame ew = new ErrorWindow(ex.getMessage());
            ew.setTitle("Error");
            ew.setLocationRelativeTo(this);
            ew.setVisible(true);
            ew.setAlwaysOnTop(true);
        }
    }//GEN-LAST:event_joinButtonMouseClicked
    
    private void hostButtonMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_hostButtonMouseClicked
        try {
            if (!isPrivateGame) {
                talkToMasterServer(isSplatPack ? "SP" : "C1", RequestType.PublishServer);
            }
            // Launch multiplayer and host a server
            mainWindow.saveChangeToConf(false, true, "");
            Process proc = mainWindow.launchGame();
            // Wait for the game to finish
            proc.waitFor();
            if (!isPrivateGame) {
                talkToMasterServer(isSplatPack ? "SP" : "C1", RequestType.RetractServer);
            }
        } catch (Exception ex){
            JFrame ew = new ErrorWindow(ex.getMessage());
            ew.setTitle("Error");
            ew.setLocationRelativeTo(this);
            ew.setVisible(true);
            ew.setAlwaysOnTop(true);
        }
    }//GEN-LAST:event_hostButtonMouseClicked

    private void serverListTableMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_serverListTableMouseClicked
        int row = serverListTable.getSelectedRow();
        joinIPTextField.setText((String) serverListTable.getModel().getValueAt(row, 0));
    }//GEN-LAST:event_serverListTableMouseClicked

    private void privateGameCheckBoxItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_privateGameCheckBoxItemStateChanged
        isPrivateGame = (evt.getStateChange() == ItemEvent.SELECTED);
    }//GEN-LAST:event_privateGameCheckBoxItemStateChanged


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton hostButton;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JButton joinButton;
    private javax.swing.JTextField joinIPTextField;
    private javax.swing.JCheckBox privateGameCheckBox;
    private javax.swing.JTable serverListTable;
    // End of variables declaration//GEN-END:variables
}
