/**
 * This file is part of Carma Launcher.
 * 
 * Carma Launcher is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * 
 * Carma Launcher is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * Carma Launcher. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2011, Casper Kejlberg-Rasmussen.
 */

/*
 * OptionsWindow.java
 *
 * Created on 10-06-2011, 23:05:31
 */
package carmalauncher;

import java.awt.event.WindowEvent;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Map;
import java.util.TreeMap;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JFrame;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

/**
 *
 * @author Casper Kejlberg-Rasmussen
 */
public class OptionsWindow extends javax.swing.JFrame {
    private MainWindow mainWindow;
    // Fields to save upon closing of the window
    private String ressolution;
    private String output;
    private String scaler;
    // Remember to save the masterServerIPField 
    
    private void initializeFromConf() throws IOException{
        Map<String, String> optionPairs = mainWindow.getFromConf(new String[]{
            "fullresolution", "output", "scaler", "masterserver"});
        ressolution = optionPairs.get("fullresolution");
        output = optionPairs.get("output");
        scaler = optionPairs.get("scaler");
        masterServerIPField.setText(optionPairs.get("masterserver"));
    }
    
    private void saveToConf() throws IOException{
        Map<String, String> optionPairs = new TreeMap<String, String>();
        optionPairs.put("fullresolution", ressolution); 
        optionPairs.put("output", output);
        optionPairs.put("scaler", scaler);
        optionPairs.put("#masterserver", masterServerIPField.getText());
        mainWindow.saveToConf(optionPairs);
    }
    
    private void setOptionsInWindow(){
        boolean ressolutionFound = false;
        Object customResItem = null;
        for(int i=0; i<ressolutionComboBox.getModel().getSize(); i++){
            Object item = ressolutionComboBox.getModel().getElementAt(i);
            if(ressolution.equals(item.toString())){
                ressolutionComboBox.getModel().setSelectedItem(item);
                ressolutionFound = true;
            }
            if(item.toString().equals("Custom")){
                customResItem = item;
            }
        }
        if(!ressolutionFound){
           customRessolutionTextField.setText(ressolution);
           ressolutionComboBox.getModel().setSelectedItem(customResItem);
        }
        for(int i=0; i<outputComboBox.getModel().getSize(); i++){
            Object item = outputComboBox.getModel().getElementAt(i);
            if(output.equals(item.toString())){
                outputComboBox.getModel().setSelectedItem(item);
            }
        }
        for(int i=0; i<scalerComboBox.getModel().getSize(); i++){
            Object item = scalerComboBox.getModel().getElementAt(i);
            if(scaler.equals(item.toString())){
                scalerComboBox.getModel().setSelectedItem(item);
            }
        }
    }
    
    /** Creates new form OptionsWindow */
    public OptionsWindow(MainWindow mw) throws IOException {
        mainWindow = mw;
        initComponents();
        initializeFromConf();
        setOptionsInWindow();
        this.addWindowListener(new java.awt.event.WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent winEvt) {
                try {
                    saveToConf();
                } catch (IOException ex) {
                    JFrame ew = new ErrorWindow(ex.getMessage());
                    ew.setTitle("Error");
                    ew.setLocationRelativeTo(null);
                    ew.setVisible(true);
                    ew.setAlwaysOnTop(true);
                }
            }
        });
        // Listen for changes in the text
        customRessolutionTextField.getDocument().addDocumentListener(new DocumentListener() {
            public void changedUpdate(DocumentEvent e) {
                if(ressolutionComboBox.getSelectedItem().toString().equals("Custom")){
                    ressolution = customRessolutionTextField.getText();
                }
            }

            public void removeUpdate(DocumentEvent e) {
                changedUpdate(e);
            }

            public void insertUpdate(DocumentEvent e) {
                changedUpdate(e);
            }
        });
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        ressolutionComboBox = new javax.swing.JComboBox();
        ressolutionLabel = new javax.swing.JLabel();
        outputLabel = new javax.swing.JLabel();
        scalerLabel = new javax.swing.JLabel();
        outputComboBox = new javax.swing.JComboBox();
        masterServerLabel = new javax.swing.JLabel();
        scalerComboBox = new javax.swing.JComboBox();
        masterServerIPField = new javax.swing.JTextField();
        customRessolutionTextField = new javax.swing.JTextField();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);

        ressolutionComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "1024x768", "1280x960", "1280x1024", "1600x1200", "1280x768", "1920x1080", "1920x1200", "Custom" }));
        ressolutionComboBox.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                ressolutionComboBoxItemStateChanged(evt);
            }
        });

        ressolutionLabel.setText("Ressolution");

        outputLabel.setText("Output");

        scalerLabel.setText("Scaler");

        outputComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "surface", "overlay", "ddraw", "opengl", "openglnb" }));
        outputComboBox.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                outputComboBoxItemStateChanged(evt);
            }
        });

        masterServerLabel.setText("Master server");

        scalerComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "none", "normal2x", "normal3x", "advmame2x", "advmame3x", "advinterp2x", "advinterp3x", "hq2x", "hq3x", "2xsai", "super2xsai", "supereagle", "tv2x", "tv3x", "rgb2x", "rgb3x", "scan2x", "scan3x" }));
        scalerComboBox.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                scalerComboBoxItemStateChanged(evt);
            }
        });

        masterServerIPField.setText("Master server ip or url");

        customRessolutionTextField.setText("Enter a custom ressolution, e.g. 1440x900");
        customRessolutionTextField.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                customRessolutionTextFieldMouseClicked(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(ressolutionLabel)
                    .addComponent(outputLabel)
                    .addComponent(scalerLabel)
                    .addComponent(masterServerLabel))
                .addGap(18, 18, 18)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(ressolutionComboBox, javax.swing.GroupLayout.Alignment.TRAILING, 0, 322, Short.MAX_VALUE)
                    .addComponent(outputComboBox, javax.swing.GroupLayout.Alignment.TRAILING, 0, 322, Short.MAX_VALUE)
                    .addComponent(scalerComboBox, javax.swing.GroupLayout.Alignment.TRAILING, 0, 322, Short.MAX_VALUE)
                    .addComponent(masterServerIPField, javax.swing.GroupLayout.DEFAULT_SIZE, 322, Short.MAX_VALUE)
                    .addComponent(customRessolutionTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 322, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(ressolutionComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(ressolutionLabel))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(customRessolutionTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(outputComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(outputLabel))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(scalerComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(scalerLabel))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(masterServerIPField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(masterServerLabel))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void ressolutionComboBoxItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_ressolutionComboBoxItemStateChanged
        if(evt.getItem().toString().equals("Custom")){
            ressolution = customRessolutionTextField.getText();
        } else {
            ressolution = evt.getItem().toString();
        }
    }//GEN-LAST:event_ressolutionComboBoxItemStateChanged

    private void outputComboBoxItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_outputComboBoxItemStateChanged
        output = (String) evt.getItem();
    }//GEN-LAST:event_outputComboBoxItemStateChanged

    private void scalerComboBoxItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_scalerComboBoxItemStateChanged
        scaler = (String) evt.getItem();
    }//GEN-LAST:event_scalerComboBoxItemStateChanged

    private void customRessolutionTextFieldMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_customRessolutionTextFieldMouseClicked
        if(customRessolutionTextField.getText().contains("Enter a custom ressolution, e.g. 1440x900")){
            customRessolutionTextField.setText("");
        }
    }//GEN-LAST:event_customRessolutionTextFieldMouseClicked

    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTextField customRessolutionTextField;
    private javax.swing.JTextField masterServerIPField;
    private javax.swing.JLabel masterServerLabel;
    private javax.swing.JComboBox outputComboBox;
    private javax.swing.JLabel outputLabel;
    private javax.swing.JComboBox ressolutionComboBox;
    private javax.swing.JLabel ressolutionLabel;
    private javax.swing.JComboBox scalerComboBox;
    private javax.swing.JLabel scalerLabel;
    // End of variables declaration//GEN-END:variables
}
