/**
 * This file is part of Carma Launcher.
 * 
 * Carma Launcher is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * 
 * Carma Launcher is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * Carma Launcher. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2011, Casper Kejlberg-Rasmussen.
 */
package masterserver;

import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.math.BigInteger;
import java.net.InetAddress;
import java.net.Socket;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * @author Casper Kejlberg-Rasmussen
 */
public class CleanUpServer implements Runnable{
    private final Map<BigInteger, ServerInfo> serverlist;
    private int cleanUpPort;
    private int cleanUpPause = 300; // In seconds
    
    public CleanUpServer(Map<BigInteger, ServerInfo> serverlist, int port){
        this.serverlist = serverlist;
        this.cleanUpPort = port;
    }
    
    public CleanUpServer(Map<BigInteger, ServerInfo> serverlist, int port, int pauseInterval){
        this(serverlist, port);
        this.cleanUpPause = pauseInterval;
    }
    
    @Override
    public void run(){
        while (true) {
            for (final ServerInfo si : serverlist.values()) {
                List<InetAddress> ips = new LinkedList<InetAddress>();
                ips.add(si.getServerIP());
                ips.addAll(si.getClients());
                for (InetAddress ip : ips) {
                    try {
                        Socket s = new Socket(ip, cleanUpPort);
                        ObjectOutputStream oos = new ObjectOutputStream(s.getOutputStream());
                        oos.writeObject(RequestType.AreYouStillAlive);
                        oos.flush();
                        ObjectInputStream ois = new ObjectInputStream(s.getInputStream());
                        RequestType rt = (RequestType) ois.readObject();
                        if (rt != RequestType.YesIAmStillAlive) {
                            serverlist.remove(si.id());
                        }
                    } catch (Exception ex) {
                        serverlist.remove(si.id());
                    }
                }
            }
            try {
                Thread.sleep(cleanUpPause * 1000);
            } catch (InterruptedException ex) {
                System.out.println("Error in the cleanUpThread\n" + ex.getMessage());
            }
        }
    }
}
